#import "AppDelegate.h"


@implementation AppDelegate

- (IBAction)setIconFilepath:(id)sender;
{
    NSOpenPanel *openPanel;
	
    openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseDirectories:NO];
    [openPanel setAllowsMultipleSelection:NO];
    [openPanel setResolvesAliases:YES];
    [openPanel setCanChooseFiles:YES];
    [openPanel setTreatsFilePackagesAsDirectories:YES];
    
    NSArray *fileTypes = [NSArray arrayWithObjects: @"icns", nil];
    
    if ([openPanel runModalForTypes:fileTypes] == NSOKButton)
    {
        [icnsFilepath setStringValue:[openPanel filename]];
    }
    
    
}

- (IBAction)setDirectoryPath:(id)sender;
{
    NSOpenPanel *panel = [NSOpenPanel openPanel];
    
    // Don't allow the user to choose files, only directories
    // for our destination.
    [panel setCanChooseFiles:NO];
    [panel setCanChooseDirectories:YES];
    
    if( [panel runModalForDirectory:nil file:nil] == NSOKButton )
    {
        [dirPath setStringValue:[panel directory]];
    }
}

- (IBAction)setIconForDirectory:(id)sender;
{
    [self setIcon:[icnsFilepath stringValue] forDirectory:[dirPath stringValue]];
    
}

- (void)setIcon:(NSString*)iconFilepath forDirectory:(NSString*)directoryPath;
{
    
    FSRef ref;
    OSStatus result;
    IconFamilyHandle hIconFamily;

    CFURLRef urlRef;
    Boolean gotFSRef;
    
    // Create a CFURL with the specified POSIX path.
    urlRef = CFURLCreateWithFileSystemPath( kCFAllocatorDefault,
                                           (CFStringRef) iconFilepath,
                                           kCFURLPOSIXPathStyle,
                                           FALSE /* isDirectory */ );
    if (urlRef == NULL)
    {
        NSLog(@"** Couldn't make a CFURLRef for the file." );
        return;
    }
    
    // Try to create an FSRef from the URL.  (If the specified file doesn't exist, this
    // function will return false, but if we've reached this code we've already insured
    // that the file exists.)
    gotFSRef = CFURLGetFSRef( urlRef, &ref );
    CFRelease( urlRef );
    
    if (!gotFSRef)
    {
        NSLog(@"** Couldn't get an FSRef for the file.\n" );
        return;
    }
    result = ReadIconFromFSRef( &ref, &hIconFamily );
    
    [self setAsCustomIconForDirectory:directoryPath familyHandle:hIconFamily];
    
}

- (BOOL) setAsCustomIconForDirectory:(NSString*)path familyHandle:(IconFamilyHandle)hIconFam;
{
    NSFileManager *fm = [NSFileManager defaultManager];
    BOOL isDir;
    BOOL exists;
    NSString *iconrPath;
    FSRef targetFolderFSRef, iconrFSRef;
    SInt16 file;
    OSErr result;
    struct HFSUniStr255 filename;
    struct FSCatalogInfo catInfo;
    Handle hExistingCustomIcon;
    IconFamilyHandle hIconFamily;
    Handle hIconFamilyCopy;
    
    hIconFamily = hIconFam;
    
    // Confirm that "path" exists and specifies a directory.
    exists = [fm fileExistsAtPath:path isDirectory:&isDir];
    if( !isDir || !exists )
        return NO;
    
    // Get an FSRef for the folder.
    
    CFURLRef urlRef;
    Boolean gotFSRef;

    
    // Create a CFURL with the specified POSIX path.
    urlRef = CFURLCreateWithFileSystemPath( kCFAllocatorDefault,
                                           (CFStringRef) path,
                                           kCFURLPOSIXPathStyle,
                                           FALSE /* isDirectory */ );
    if (urlRef == NULL)
    {
        NSLog(@"** Couldn't make a CFURLRef for the file." );
        return NO;
    }
    
    // Try to create an FSRef from the URL.  (If the specified file doesn't exist, this
    // function will return false, but if we've reached this code we've already insured
    // that the file exists.)
    gotFSRef = CFURLGetFSRef( urlRef, &targetFolderFSRef );
    CFRelease( urlRef );
    
    if (!gotFSRef)
    {
        NSLog(@"** Couldn't get an FSRef for the file." );
        return NO;
    }
    
        
    
    // Remove and re-create any existing "Icon\r" file in the directory, and get an FSRef for it.
    iconrPath = [path stringByAppendingPathComponent:@"Icon\r"];
    if( [fm fileExistsAtPath:iconrPath] )
    {
        if( ![fm removeFileAtPath:iconrPath handler:nil] )
            return NO;
    }
    
    if (![@"" writeToFile:iconrPath atomically:YES])
    {
        return NO;
    }
    
    
    
    
    
    // Create a CFURL with the specified POSIX path.
    urlRef = CFURLCreateWithFileSystemPath( kCFAllocatorDefault,
                                           (CFStringRef) iconrPath,
                                           kCFURLPOSIXPathStyle,
                                           FALSE /* isDirectory */ );
    if (urlRef == NULL)
    {
        NSLog(@"** Couldn't make a CFURLRef for the file." );
        return NO;
    }
    
    // Try to create an FSRef from the URL.  (If the specified file doesn't exist, this
    // function will return false, but if we've reached this code we've already insured
    // that the file exists.)
    gotFSRef = CFURLGetFSRef( urlRef, &iconrFSRef );
    CFRelease( urlRef );
    
    if (!gotFSRef)
    {
        NSLog(@"** Couldn't get an FSRef for the file.\n" );
        return NO;
    }
    

    
    
    
    // Get type and creator information for the Icon file.
    result = FSGetCatalogInfo(
                              &iconrFSRef,
                              kFSCatInfoFinderInfo,
                              &catInfo,
                              /*outName*/ NULL,
                              /*fsSpec*/ NULL,
                              /*parentRef*/ NULL );
    if( result == fnfErr ) {
        // The file doesn't exist. Prepare to create it.
        
        struct FileInfo *finderInfo = (struct FileInfo *)catInfo.finderInfo;
        
        // These are the file type and creator given to Icon files created by
        // the Finder.
        finderInfo->fileType = 'icon';
        finderInfo->fileCreator = 'MACS';
        
        // Icon files should be invisible.
        finderInfo->finderFlags = kIsInvisible;
        
        // Because the inited flag is not set in finderFlags above, the Finder
        // will ignore the location, unless it's in the 'magic rectangle' of
        // { -24,000, -24,000, -16,000, -16,000 } (technote TB42).
        // So we need to make sure to set this to zero anyway, so that the
        // Finder will position it automatically. If the user makes the Icon
        // file visible for any reason, we don't want it to be positioned in an
        // exotic corner of the window.
        finderInfo->location.h = finderInfo->location.v = 0;
        
        // Standard reserved-field practice.
        finderInfo->reservedField = 0;
        
        // Modified by Matteo Rattotti on: Wed Dec  5 14:56:52 2007
        // the catalog must be set
        FSSetCatalogInfo (&iconrFSRef, kFSCatInfoFinderInfo, &catInfo);
        
    } else {
		if( result != noErr ){
			return NO;
		}
		else{ // Modified by Matteo Rattotti on: Wed Dec  5 14:56:52 2007
			// File already exist, so we only need to set the flag we want (invisib, etc...)
			struct FileInfo *finderInfo = (struct FileInfo *)catInfo.finderInfo;
			
			// File type
			finderInfo->fileType = 'icon';
			finderInfo->fileCreator = 'MACS';
            
			// Icon files should be invisible.
			finderInfo->finderFlags = kIsInvisible;
			finderInfo->location.h = finderInfo->location.v = 0;
            
			// Standard reserved-field practice.
			finderInfo->reservedField = 0;
            
			FSSetCatalogInfo (&iconrFSRef, kFSCatInfoFinderInfo, &catInfo);
		}
	}
    
    // Get the filename, to be applied to the Icon file.
    filename.length = [@"Icon\r" length];
    [@"Icon\r" getCharacters:filename.unicode];
    
    // Make sure the file has a resource fork that we can open.  (Although
    // this sounds like it would clobber an existing resource fork, the Carbon
    // Resource Manager docs for this function say that's not the case.)
    FSCreateResFile(
                    &targetFolderFSRef,
                    filename.length,
                    filename.unicode,
                    kFSCatInfoFinderInfo,
                    &catInfo,
                    &iconrFSRef,
                    /*newSpec*/ NULL);
    result = ResError();
    if (!(result == noErr || result == dupFNErr))
        return NO;
    
    // Open the file's resource fork.
    file = FSOpenResFile( &iconrFSRef, fsRdWrPerm );
    if (file == -1)
        return NO;
    
    // Make a copy of the icon family data to pass to AddResource().
    // (AddResource() takes ownership of the handle we pass in; after the
    // CloseResFile() call its master pointer will be set to 0xffffffff.
    // We want to keep the icon family data, so we make a copy.)
    // HandToHand() returns the handle of the copy in hIconFamily.
    hIconFamilyCopy = (Handle) hIconFamily;
    result = HandToHand( &hIconFamilyCopy );
    if (result != noErr) {
        CloseResFile( file );
        return NO;
    }
    
    // Remove the file's existing kCustomIconResource of type kIconFamilyType
    // (if any).
    hExistingCustomIcon = GetResource( kIconFamilyType, kCustomIconResource );
    if( hExistingCustomIcon )
        RemoveResource( hExistingCustomIcon );
    
    // Now add our icon family as the file's new custom icon.
    AddResource( (Handle)hIconFamilyCopy, kIconFamilyType,
                kCustomIconResource, "\p");
    
    if (ResError() != noErr) {
        CloseResFile( file );
        return NO;
    }
    
    // Close the file's resource fork, flushing the resource map and new icon
    // data out to disk.
    CloseResFile( file );
    if (ResError() != noErr)
        return NO;
    
    result = FSGetCatalogInfo( &targetFolderFSRef,
                              kFSCatInfoFinderInfo,
                              &catInfo,
                              /*outName*/ NULL,
                              /*fsSpec*/ NULL,
                              /*parentRef*/ NULL);
    if( result != noErr )
        return NO;
    
    // Tell the Finder that the folder now has a custom icon.
    ((struct FolderInfo *)catInfo.finderInfo)->finderFlags = ( ((struct FolderInfo *)catInfo.finderInfo)->finderFlags | kHasCustomIcon ) & ~kHasBeenInited;
    
    result = FSSetCatalogInfo( &targetFolderFSRef,
                              kFSCatInfoFinderInfo,
                              &catInfo);
    if( result != noErr )
        return NO;
    
    // Notify the system that the target directory has changed, to give Finder
    // the chance to find out about its new custom icon.
    result = FNNotify( &targetFolderFSRef, kFNDirectoryModifiedMessage, kNilOptions );
    if (result != noErr)
        return NO;
	
    return YES;
}


@end